@@ -45,7 +45,7 @@ def clean_operations(operations, openapi_major_version):
         Otherwise, it is assumed to be a reference name as string and the corresponding $ref
         string is returned.
 
-        :param str obj_type: 'parameter' or 'response'
+        :param str obj_type: "parameter" or "response"
         :param dict|str obj: parameter or response in dict form or as ref_id string
         :param int openapi_major_version: The major version of the OpenAPI standard
         """
@@ -67,10 +67,12 @@ def clean_operations(operations, openapi_major_version):
                 get_ref("parameter", p, openapi_major_version) for p in parameters
             ]
         if "responses" in operation:
+            responses = OrderedDict()
             for code, response in iteritems(operation["responses"]):
-                operation["responses"][code] = get_ref(
+                responses[str(code)] = get_ref(
                     "response", response, openapi_major_version
                 )
+            operation["responses"] = responses
 
 
 class Components(object):
@@ -98,6 +100,7 @@ class Components(object):
         return {
             COMPONENT_SUBSECTIONS[self.openapi_version.major][k]: v
             for k, v in iteritems(subsections)
+            if v != {}
         }
 
     def schema(self, name, component=None, **kwargs):
@@ -237,15 +240,18 @@ class APISpec(object):
     def to_dict(self):
         ret = {
             "paths": self._paths,
-            "tags": self._tags,
             "info": {"title": self.title, "version": self.version},
         }
+        if self._tags:
+            ret["tags"] = self._tags
         if self.openapi_version.major < 3:
             ret["swagger"] = self.openapi_version.vstring
             ret.update(self.components.to_dict())
         else:
             ret["openapi"] = self.openapi_version.vstring
-            ret.update({"components": self.components.to_dict()})
+            components_dict = self.components.to_dict()
+            if components_dict:
+                ret["components"] = components_dict
         ret = deepupdate(ret, self.options)
         return ret
 
